@@ -90,8 +90,7 @@ fn spawn_server(b: Arc<Barrier>) -> JoinHandle<()> {
 fn get_hello() -> Message {
   use kwap_msg::*;
   Message { id: Id(1),
-            ty: Type(0),
-            __optc: Default::default(),
+            ty: Type::Con,
             ver: Default::default(),
             token: Token(Default::default()),
             code: Code { class: 0, detail: 1 }, // GET
@@ -103,9 +102,8 @@ fn get_hello() -> Message {
 fn ok_hello(token: kwap_msg::Token) -> Message {
   use kwap_msg::*;
   Message { id: Id(1),
-            ty: Type(2), // ACK
+            ty: Type::Ack, // ACK
             ver: Default::default(),
-            __optc: Default::default(),
             token,
             code: Code { class: 2, detail: 5 }, // 2.05 OK
             opts: Vec::new(),
@@ -115,9 +113,8 @@ fn ok_hello(token: kwap_msg::Token) -> Message {
 fn not_found(token: kwap_msg::Token) -> Message {
   use kwap_msg::*;
   Message { id: Id(1),
-            ty: Type(2), // ACK
+            ty: Type::Ack, // ACK
             ver: Default::default(),
-            __optc: Default::default(),
             token,
             code: Code { class: 4, detail: 4 }, // 4.04 NOT FOUND
             opts: Vec::new(),
