@@ -64,7 +64,7 @@
 //! ![chart](https://raw.githubusercontent.com/clov-coffee/kwap/main/kwap_msg/docs/to_bytes.svg)
 //! </details>
 
-#![doc(html_root_url = "https://docs.rs/kwap-msg/0.2.7")]
+#![doc(html_root_url = "https://docs.rs/kwap-msg/0.4.0")]
 #![cfg_attr(all(not(test), feature = "no_std"), no_std)]
 #![cfg_attr(not(test), forbid(missing_debug_implementations, unreachable_pub))]
 #![cfg_attr(not(test), deny(unsafe_code, missing_copy_implementations))]
@@ -99,7 +99,7 @@ pub use to_bytes::TryIntoBytes;
 
 #[doc = rfc_7252_doc!("5.5")]
 #[derive(Clone, Debug, PartialEq, PartialOrd)]
-pub struct Payload<C: Array<u8>>(pub C) where for<'a> &'a C: IntoIterator<Item = &'a u8>;
+pub struct Payload<C: Array<Item = u8>>(pub C);
 
 /// Message that uses Vec byte buffers
 #[cfg(feature = "alloc")]
@@ -156,23 +156,18 @@ pub type ArrayVecMessage<const PAYLOAD_CAP: usize, const N_OPTS: usize, const OP
 ///
 /// let expected = VecMessage {
 ///   id: Id(1),
-///   ty: Type(0),
+///   ty: Type::Con,
 ///   ver: Version(1),
 ///   token: Token(tinyvec::array_vec!([u8; 8] => 254)),
 ///   opts: opts_expected,
 ///   code: Code {class: 2, detail: 5},
 ///   payload: Payload(b"hello, world!".to_vec()),
-///   __optc: Default::default(),
 /// };
 ///
 /// assert_eq!(msg, expected);
 /// ```
 #[derive(Clone, PartialEq, PartialOrd, Debug)]
-pub struct Message<PayloadC: Array<u8>, OptC: Array<u8> + 'static, Opts: Array<Opt<OptC>>>
-  where for<'a> &'a PayloadC: IntoIterator<Item = &'a u8>,
-        for<'a> &'a OptC: IntoIterator<Item = &'a u8>,
-        for<'a> &'a Opts: IntoIterator<Item = &'a Opt<OptC>>
-{
+pub struct Message<PayloadC: Array<Item = u8>, OptC: Array<Item = u8> + 'static, Opts: Array<Item = Opt<OptC>>> {
   /// see [`Id`] for details
   pub id: Id,
   /// see [`Type`] for details
@@ -187,21 +182,15 @@ pub struct Message<PayloadC: Array<u8>, OptC: Array<u8> + 'static, Opts: Array<O
   pub opts: Opts,
   /// see [`Payload`]
   pub payload: Payload<PayloadC>,
-  /// empty field using the Opt internal byte collection type
-  pub __optc: core::marker::PhantomData<OptC>,
 }
 
-impl<P: Array<u8>, O: Array<u8>, Os: Array<Opt<O>>> GetSize for Message<P, O, Os>
-  where for<'b> &'b P: IntoIterator<Item = &'b u8>,
-        for<'b> &'b O: IntoIterator<Item = &'b u8>,
-        for<'b> &'b Os: IntoIterator<Item = &'b Opt<O>>
-{
+impl<P: Array<Item = u8>, O: Array<Item = u8>, Os: Array<Item = Opt<O>>> GetSize for Message<P, O, Os> {
   fn get_size(&self) -> usize {
     let header_size = 4;
     let payload_marker_size = 1;
     let payload_size = self.payload.0.get_size();
     let token_size = self.token.0.len();
-    let opts_size: usize = (&self.opts).into_iter().map(|o| o.get_size()).sum();
+    let opts_size: usize = self.opts.iter().map(|o| o.get_size()).sum();
 
     header_size + payload_marker_size + payload_size + token_size + opts_size
   }
@@ -244,15 +233,37 @@ pub(crate) struct Byte1 {
 #[derive(Copy, Clone, PartialEq, PartialOrd, Debug)]
 pub struct Id(pub u16);
 
-/// Message type:
-/// - 0 Confirmable; "Please let me know when you received this"
-/// - 1 Non-confirmable; "I don't care if this gets to you"
-/// - 2 Acknowledgement; "I got your message!"
-/// - 3 Reset; ""
+/// Indicates if this message is of
+/// type Confirmable (0), Non-confirmable (1), Acknowledgement (2), or Reset (3).
 ///
 /// See [RFC7252 - Message Details](https://datatracker.ietf.org/doc/html/rfc7252#section-3) for context
-#[derive(Copy, Clone, PartialEq, PartialOrd, Debug)]
-pub struct Type(pub u8);
+#[derive(Copy, Clone, Eq, Ord, PartialEq, PartialOrd, Debug)]
+pub enum Type {
+  /// Some messages do not require an acknowledgement.  This is
+  /// particularly true for messages that are repeated regularly for
+  /// application requirements, such as repeated readings from a sensor.
+  Non,
+  /// Some messages require an acknowledgement.  These messages are
+  /// called "Confirmable".  When no packets are lost, each Confirmable
+  /// message elicits exactly one return message of type Acknowledgement
+  /// or type Reset.
+  Con,
+  /// An Acknowledgement message acknowledges that a specific
+  /// Confirmable message arrived.  By itself, an Acknowledgement
+  /// message does not indicate success or failure of any request
+  /// encapsulated in the Confirmable message, but the Acknowledgement
+  /// message may also carry a Piggybacked Response.
+  Ack,
+  /// A Reset message indicates that a specific message (Confirmable or
+  /// Non-confirmable) was received, but some context is missing to
+  /// properly process it.  This condition is usually caused when the
+  /// receiving node has rebooted and has forgotten some state that
+  /// would be required to interpret the message.  Provoking a Reset
+  /// message (e.g., by sending an Empty Confirmable message) is also
+  /// useful as an inexpensive check of the liveness of an endpoint
+  /// ("CoAP ping").
+  Reset,
+}
 
 /// Version of the CoAP protocol that the message adheres to.
 ///
@@ -289,13 +300,12 @@ pub(crate) fn test_msg() -> (VecMessage, Vec<u8>) {
   opts.push(opt);
 
   let msg = VecMessage { id: Id(1),
-                         ty: Type(0),
+                         ty: Type::Con,
                          ver: Version(1),
                          token: Token(tinyvec::array_vec!([u8; 8] => 254)),
                          opts,
                          code: Code { class: 2, detail: 5 },
-                         payload: Payload(b"hello, world!".into_iter().copied().collect()),
-                         __optc: Default::default() };
+                         payload: Payload(b"hello, world!".into_iter().copied().collect()) };
   (msg, bytes)
 }
 
